package com.ekingwin.bas.cloud.document.web;

import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import com.ekingwin.bas.cloud.document.dto.UploadFileDto;
import com.ekingwin.bas.cloud.document.service.IIAttachmentService;
import com.ekingwin.bas.cloud.infra.utils.web.WebUtils;
import com.ekingwin.bas.cloud.infra.web.BaseController;

import io.swagger.annotations.Api;

/**
 * 附件管理控制器，提供对于附件操作的请求的访问控制
 * @author chenjb
 *
 */  
@RestController
@RequestMapping("/attach")
@Api("附件上传下载控制器")
public class AttachmentController extends BaseController {


	@Autowired
	private IIAttachmentService attachmentManager;
	
	/**
	 * 上传文件
	 */
	@RequestMapping(value="/upload", method = RequestMethod.POST)
	public UploadFileDto upload(@RequestParam(value = "file") MultipartFile file, @RequestParam(value = "name", required=false)String name) throws Exception{
		Map<String, String[]> fileMeta = WebUtils.getRequest().getParameterMap();
		if(StringUtils.isEmpty(name)){
			name = file.getOriginalFilename();
		}
		UploadFileDto uf = attachmentManager.upload(file.getInputStream(), name, fileMeta);
		return uf;
	}
	
	/**
	 * 下载文档
	 */
	@RequestMapping(value="/download/{id}", method = RequestMethod.GET) 
	public ResponseEntity<byte[]> download(@PathVariable(value = "id")String id) throws Exception{
		return attachmentManager.downloadFileByte(id);
	}
    
	/**
	 * 批量下载文档
	 * @return
	 * @throws Exception
	 */
	@RequestMapping(value="/download/batch/{ids}", method = RequestMethod.GET)
	public ResponseEntity<byte[]> batchDownload(@PathVariable(value = "ids")String ids) throws Exception{
		return attachmentManager.downloadFileBatchByte(ids.split(","));
	}
	
	/**
	 * 删除附件
	 * @return
	 * @throws Exception
	 */
	@RequestMapping(value="/delete/{id}", method = RequestMethod.GET) 
	public String delete(@PathVariable(value = "id")String id) throws Exception{
		attachmentManager.deleteFile(id);
		return "附件已删除";
	}
	
	
}
