package com.ekingwin.bas.cloud.org.service;

import java.util.List;
import java.util.Map;

import com.ekingwin.bas.cloud.infra.dto.BaseResult;
import com.ekingwin.bas.cloud.infra.dto.QueryDTO;
import com.ekingwin.bas.cloud.infra.exception.BasException;
import com.ekingwin.bas.cloud.infra.service.IBaseCurdService;
import com.ekingwin.bas.cloud.infra.utils.db.PageCriteria;
import com.ekingwin.bas.cloud.org.OrganizationException;
import com.ekingwin.bas.cloud.org.dao.entity.Organization;
import com.ekingwin.bas.cloud.org.dto.OrganizationDto;
import com.ekingwin.bas.cloud.org.dto.OrganizationTreeDto;
import com.ekingwin.bas.cloud.org.dto.OrganizationTreePostDto;
import com.ekingwin.bas.cloud.org.dto.PostDto;
import com.ekingwin.bas.cloud.user.UserException;
import com.ekingwin.bas.cloud.user.dto.UserDto;

public interface IOrganizationService extends IBaseCurdService<OrganizationDto> {

	/**
	 * 获取指定用户的部门信息
	 * @param userId
	 * @return
	 * @throws UserException 
	 */
	public OrganizationDto getUserDept(String userId) throws OrganizationException;
	
	/**
	 * 获取指定用户的岗位信息
	 * @param userId
	 * @return
	 * @throws UserException 
	 */
	public List<OrganizationDto> getUserPost(String userId) throws OrganizationException;
	/**
	 * 状态删除
	 * @param id
	 * @return
	 */
	public boolean deleteById(String id) throws OrganizationException;
	
	//查询第一级组织树
	public List<OrganizationTreeDto> selectFirstTree() throws OrganizationException;
	
	//查询所有父级树
    public List<OrganizationTreeDto> getPartentTree(String id)throws OrganizationException;
    //查询顶级父级树
    public List<OrganizationTreeDto> getLastParentTree()throws OrganizationException;
    public List<OrganizationTreeDto> getChildTree(String id)throws OrganizationException;
    //根据id查询所有下一级组织树
    public List<OrganizationTreeDto> getChildTreeById(String id)throws OrganizationException;
   //根据id查询所有下级组织树
    public List<OrganizationTreeDto> getAllChildTree(String id)throws OrganizationException;
    public String getStringById(String id)throws OrganizationException;
    
    public List<PostDto> getPostById(String userid) throws OrganizationException;
    
    public List<OrganizationTreeDto>  getChildPost(Map<String, String> map) throws OrganizationException;
    public PageCriteria<PostDto> queryPostById(QueryDTO query) throws OrganizationException;
    /**
	 * 通过工号；获取该工号从属的所有组织的id，用逗号分隔
	 * @param empuid
	 * @return
	 * @throws OrganizationException
	 */
	public String getOrgIdsByUserId(String empuid)throws OrganizationException;

	/**
	 * 查询组织维度下面的岗位组织
	 * @param query
	 * @return
	 * @throws OrganizationException 
	 */
	public List<OrganizationTreePostDto> getPostByOrgid(String logid,String orgid,int num) throws OrganizationException;

	/**
	 * 通过点击的组织ID和登录用户ID,获取该组织下的所有员工
	 * @param query
	 * @param logid
	 * @param orgid
	 * @return
	 * @throws UserException 
	 */
	public PageCriteria<UserDto> getUserByLogIdAndOrgid(QueryDTO query, String logid, String orgid) throws UserException;

	/**
	 * 根据岗位id获取岗位全路径
	 * @param postid
	 * @return
	 */
	public String getPostRouteByPostid(String postid);
	
	/**
	 * getuserByOrg
	 * @param query
	 * @param logid
	 * @param orgid
	 * @return
	 * @throws UserException 
	 * @throws Exception 
	 */
	public BaseResult<PageCriteria<UserDto>> getUsersByOrg(QueryDTO query) throws BasException;

	/**
	 * 同步单个组织数据，用户给其他系统提供接口调用
	 * @param orgdto
	 * @throws BasException
	 */
	public BaseResult<OrganizationDto> syncOrg(OrganizationDto orgdto) throws OrganizationException;
	 
	public Organization getNewOrg(OrganizationDto orgdto) throws OrganizationException;
	
	
	/**
	 * 批量更新
	 */
	public void batchUpdate(List<Organization> updateList) throws OrganizationException;
	/**
	 * 批量新增
	 */
	public void batchInsert(List<Organization> insertList) throws OrganizationException;

}
