package com.ekingwin.bas.cloud.org.service.impl;


import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import com.ekingwin.bas.cloud.infra.dto.BaseResult;
import com.ekingwin.bas.cloud.infra.dto.FilterDTO;
import com.ekingwin.bas.cloud.infra.dto.QueryDTO;
import com.ekingwin.bas.cloud.infra.exception.BasException;
import com.ekingwin.bas.cloud.infra.service.BaseService;
import com.ekingwin.bas.cloud.infra.utils.common.ResultUtils;
import com.ekingwin.bas.cloud.infra.utils.db.PageCriteria;
import com.ekingwin.bas.cloud.org.OrganizationException;
import com.ekingwin.bas.cloud.org.dao.IOrganizationManagerDao;
import com.ekingwin.bas.cloud.org.dto.OrganizationDto;
import com.ekingwin.bas.cloud.org.dto.OrganizationUserDto;
import com.ekingwin.bas.cloud.org.dto.PostDto;
import com.ekingwin.bas.cloud.org.service.IOrganizationService;
import com.ekingwin.bas.cloud.org.service.IPostService;

@Service
public class PostServiceImpl extends BaseService implements IPostService{
	
	@Autowired
	@Qualifier("organizationServiceImpl")
	private IOrganizationService  organizationService;

	@Autowired
	private IOrganizationManagerDao  iogDao;
	
	/**  
	* @Title: save  
	* @Description:保存岗位
	* @param 
	* @return
	*/
	@Override
	public BaseResult<PostDto> save(PostDto postDto) throws BasException {
		OrganizationDto orgDto = this.postDtoToOrgDto(postDto);
		
		orgDto.setType(OrganizationDto.TYPE_POST);
		organizationService.save(orgDto);	
		PostDto newDto = new PostDto();
		newDto.setId(postDto.getId());
		return ResultUtils.build(newDto);
	}

	/**  
	* @Title: delete  
	* @Description: 根据id删除岗位
	* @param 
	* @return
	*/
	@Override
	public BaseResult delete(String id) throws BasException {
		organizationService.delete(id);
		return ResultUtils.build();
	}


	/**  
	* @Title: query  
	* @Description: 分页查询
	* @param 
	* @return
	*/
	@Override
	public BaseResult<PageCriteria<PostDto>> query(QueryDTO query) throws BasException {
		query.addFilter(new FilterDTO("type", OrganizationDto.TYPE_POST));
		PageCriteria<OrganizationDto> orgPage = organizationService.query(query).getData();
		List<OrganizationDto> orgList = orgPage.getList();
		List<PostDto> postList = new ArrayList<PostDto>();
		for(OrganizationDto orgDto:orgList){
			
			PostDto postDto = this.orgDtoToPostDto(orgDto);
			postList.add(postDto);
		}
		PageCriteria<PostDto> postPage = new PageCriteria<PostDto>();
		try {
			postPage.build(orgPage, postList);
		} catch (IllegalAccessException | InvocationTargetException e) {
			e.printStackTrace();
		}
		return ResultUtils.build(postPage);
	}

	/**  
	* @Title: get  
	* @Description: 根据id查询岗位
	* @param 
	* @return
	*/
	@Override
	public BaseResult<PostDto> get(String id) throws BasException {
		return ResultUtils.build(this.orgDtoToPostDto(organizationService.get(id).getData()));
		
	}
	
	/**
	 * PostDto转换为OrganizationDto
	 * @param postDto
	 * @return
	 */
	private OrganizationDto postDtoToOrgDto(PostDto postDto){
		OrganizationDto orgDto = new OrganizationDto();
		if(!StringUtils.isEmpty(postDto.getId())){
			orgDto.setId(postDto.getId());
		}
		orgDto.setFullname(postDto.getFullname());
		orgDto.setShortname(postDto.getShortname());
		orgDto.setCode(postDto.getPostcode());
		orgDto.setPid(postDto.getSoid());
		orgDto.setName(postDto.getName());
		orgDto.setHrtype(postDto.getPosttype());
		
		return orgDto;
	}
	
	/**
	 * OrganizationDto转换为PostDto
	 * @param orgDto
	 * @return
	 */
	private PostDto orgDtoToPostDto(OrganizationDto orgDto){
    	PostDto postDto = new PostDto();
    	
    	postDto.setId(orgDto.getId());
    	
    	postDto.setFullname(orgDto.getFullname());
    	postDto.setShortname(orgDto.getShortname());
    	postDto.setPostcode(orgDto.getCode());
    	postDto.setSoid(orgDto.getPid());
    	postDto.setName(orgDto.getName());
    	postDto.setPosttype(orgDto.getHrtype());
    	
    	try {
			postDto.setShortname(organizationService.get(orgDto.getPid()).getData().getName());
			postDto.setPcode(organizationService.get(orgDto.getPid()).getData().getCode());
		} catch (BasException e) {
			e.printStackTrace();
		}
    	
    	return postDto;
    }

	@Override
	public List<Map<String, Object>> getPostId(String id) {
		
		return iogDao.getPostId(id);
	}
	/**
	 * OrganizationDto转换为PostDto
	 * 
	 * @param orgDto
	 * @return
	 */
	private PostDto convertOrgToPost(OrganizationDto orgDto) {
		PostDto postDto = new PostDto();

		postDto.setId(orgDto.getId());

		postDto.setFullname(orgDto.getFullname());
		postDto.setShortname(orgDto.getShortname());
		postDto.setPcode(orgDto.getCode());

		postDto.setSoname(orgDto.getName());

		return postDto;
	}
	@Override
	public List<PostDto> getUserPost(String userId) throws OrganizationException {
		List<PostDto> postList = new ArrayList<PostDto>();
		List<OrganizationDto> orgList = organizationService.getUserPost(userId);
		for (OrganizationDto org : orgList) {
			postList.add(convertOrgToPost(org));
		}
		return postList;
	}

  }
    
    


	
	
	
